package com.sunyard.chsm.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.sunyard.chsm.dto.KeyInfoDTO;
import com.sunyard.chsm.enums.KeyCategory;
import com.sunyard.chsm.enums.KeyStatus;
import com.sunyard.chsm.mapper.KeyInfoMapper;
import com.sunyard.chsm.mapper.KeyTemplateMapper;
import com.sunyard.chsm.mapper.SpKeyRecordMapper;
import com.sunyard.chsm.model.entity.KeyInfo;
import com.sunyard.chsm.model.entity.KeyRecord;
import com.sunyard.chsm.model.entity.KeyTemplate;
import com.sunyard.chsm.sdf.SdfApiService;
import com.sunyard.chsm.sdf.model.EccKey;
import com.sunyard.chsm.service.KeyInfoService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.binary.Hex;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.time.temporal.ChronoUnit;

/**
 * @author liulu
 * @since 2024/10/23
 */
@Slf4j
@Service
public class KeyInfoServiceImpl implements KeyInfoService {

    @Resource
    private KeyInfoMapper keyInfoMapper;
    @Resource
    private SpKeyRecordMapper spKeyRecordMapper;
    @Resource
    private KeyTemplateMapper keyTemplateMapper;
    @Resource
    private SdfApiService sdfApiService;


    @Transactional
    @Override
    public Long save(KeyInfoDTO.Save save) {

        KeyTemplate keyTemplate = keyTemplateMapper.selectOne(
                new LambdaQueryWrapper<KeyTemplate>()
                        .eq(KeyTemplate::getCode, save.getKeyTemplateCode())
        );
        Assert.notNull(keyTemplate, "密钥模版不存在");

        LocalDateTime now = LocalDateTime.now();

        for (int i = 0; i < save.getGenNumber(); i++) {

            // 密钥信息
            KeyInfo info = new KeyInfo();
            info.setId(IdWorker.getId());
            info.setApplicationId(save.getApplicationId());
            info.setKeyTemplateId(keyTemplate.getId());
            info.setCode(String.valueOf(info.getId()));
            info.setKeyType(keyTemplate.getKeyType());
            info.setKeyAlg(keyTemplate.getKeyAlg());
            info.setKeyLength(keyTemplate.getKeyLength());
            info.setKeyUsage(keyTemplate.getKeyUsage());
            info.setCheckAlg(keyTemplate.getCheckAlg());
            info.setCreateTime(LocalDateTime.now());

            info.setStatus(KeyStatus.ENABLED.getCode());

            ChronoUnit startUnit = ChronoUnit.valueOf(keyTemplate.getStartAfterCreateUnit().toUpperCase());
            info.setEffectiveTime(now.plusSeconds(startUnit.getDuration().getSeconds() * keyTemplate.getStartAfterCreateTime()));

            ChronoUnit validUnit = ChronoUnit.valueOf(keyTemplate.getValidUnit().toUpperCase());
            info.setExpiredTime(info.getEffectiveTime().plusSeconds(validUnit.getDuration().getSeconds() * keyTemplate.getValidTime()));

            // 具体使用的密钥值
            KeyRecord record = new KeyRecord();
            record.setId(IdWorker.getId());
            record.setKeyId(info.getId());
            record.setKeyIndex(String.valueOf(record.getId()));
            record.setCheckAlg(keyTemplate.getCheckAlg());
            record.setEffectiveTime(info.getEffectiveTime());
            record.setExpiredTime(info.getExpiredTime());
            record.setCreateTime(LocalDateTime.now());

            if (KeyCategory.SYM_KEY.getCode().equals(keyTemplate.getKeyType())) {
                byte[] symKey = sdfApiService.generateRandom(16);
                record.setKeyData(Hex.encodeHexString(symKey));
                String checkHash = Hex.encodeHexString(sdfApiService.hash(symKey));
                record.setCheckValue(checkHash);
                info.setCheckValue(checkHash);
            } else {
                EccKey eccKey = sdfApiService.genKeyPairEcc();
                byte[] d = eccKey.getPriKey().getD();
                record.setKeyData(Hex.encodeHexString(d));
                String checkHash = Hex.encodeHexString(sdfApiService.hash(d));
                record.setCheckValue(checkHash);
                info.setCheckValue(checkHash);

                byte[] pubKeyBytes = eccKey.getPubKey().getPubKeyBytes();
                record.setPubKey(Hex.encodeHexString(pubKeyBytes));
            }
            keyInfoMapper.insert(info);
            spKeyRecordMapper.insert(record);
        }

        return 0L;
    }
}
